@@ -31,6 +31,7 @@ SOFTWARE.
 #include "etl/memory.h"
 #include "etl/list.h"
 #include "etl/debug_count.h"
+#include "etl/endianness.h"
 
 #include "data.h"
 
@@ -1176,9 +1177,9 @@ namespace
       uint32_t src[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67234501, 0x45016723, 0x01324576, 0x76453201 };
       uint32_t dst[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
-      etl::mem_copy(src, src + 8, dst);
-
+      uint32_t* result = etl::mem_copy(src, src + 8, dst);
       CHECK(std::equal(src, src + 8, dst));
+      CHECK(result == dst);
     }
 
     //*************************************************************************
@@ -1187,9 +1188,9 @@ namespace
       const uint32_t src[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67234501, 0x45016723, 0x01324576, 0x76453201 };
       uint32_t dst[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
-      etl::mem_copy(src, src + 8, dst);
-
+      uint32_t* result = etl::mem_copy(src, src + 8, dst);
       CHECK(std::equal(src, src + 8, dst));
+      CHECK(result == dst);
     }
 
     //*************************************************************************
@@ -1198,8 +1199,9 @@ namespace
       uint32_t src[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67234501, 0x45016723, 0x01324576, 0x76453201 };
       uint32_t dst[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
-      etl::mem_copy(src, 8, dst);
+      uint32_t* result = etl::mem_copy(src, 8, dst);
       CHECK(std::equal(src, src + 8, dst));
+      CHECK(result == dst);
     }
 
     //*************************************************************************
@@ -1208,8 +1210,9 @@ namespace
       const uint32_t src[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67234501, 0x45016723, 0x01324576, 0x76453201 };
       uint32_t dst[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
 
-      etl::mem_copy(src, 8, dst);
+      uint32_t* result = etl::mem_copy(src, 8, dst);
       CHECK(std::equal(src, src + 8, dst));
+      CHECK(result == dst);
     }
 
     //*************************************************************************
@@ -1218,9 +1221,9 @@ namespace
       uint32_t expected[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67234501, 0x45016723, 0x01324576, 0x76453201 };
       uint32_t data[12]    = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67234501, 0x45016723, 0x01324576, 0x76453201, 0, 0, 0, 0 };
 
-      etl::mem_move(data, data + 8, data + 4);
-
+      uint32_t* result = etl::mem_move(data, data + 8, data + 4);
       CHECK(std::equal(expected, expected + 8, data + 4));
+      CHECK(result == data + 4);
     }
 
     //*************************************************************************
@@ -1231,9 +1234,9 @@ namespace
       const uint32_t* data_begin = &data[0];
       const uint32_t* data_end = &data[8];
 
-      etl::mem_move(data_begin, data_end, data + 4);
-
+      uint32_t* result = etl::mem_move(data_begin, data_end, data + 4);
       CHECK(std::equal(expected, expected + 8, data + 4));
+      CHECK(result == data + 4);
     }
 
     //*************************************************************************
@@ -1242,9 +1245,9 @@ namespace
       uint32_t expected[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67234501, 0x45016723, 0x01324576, 0x76453201 };
       uint32_t data[12]    = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67234501, 0x45016723, 0x01324576, 0x76453201, 0, 0, 0, 0 };
 
-      etl::mem_move(data, 8, data + 4);
-
+      uint32_t* result = etl::mem_move(data, 8, data + 4);
       CHECK(std::equal(expected, expected + 8, data + 4));
+      CHECK(result == data + 4);
     }
 
     //*************************************************************************
@@ -1253,9 +1256,10 @@ namespace
       uint32_t expected[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67234501, 0x45016723, 0x01324576, 0x76453201 };
       uint32_t data[12]    = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67234501, 0x45016723, 0x01324576, 0x76453201, 0, 0, 0, 0 };
       const uint32_t* data_begin = &data[0];
-      etl::mem_move(data_begin, 8, data + 4);
-
+      
+      uint32_t* result = etl::mem_move(data_begin, 8, data + 4);
       CHECK(std::equal(expected, expected + 8, data + 4));
+      CHECK(result == data + 4);
     }
 
     //*************************************************************************
@@ -1342,7 +1346,7 @@ namespace
       uint32_t data[8]     = { 0, 0, 0, 0, 0, 0, 0, 0 };
       uint32_t expected[8] = { 0, 0x5A5A5A5A, 0x5A5A5A5A, 0x5A5A5A5A, 0x5A5A5A5A, 0, 0, 0 };
 
-      etl::mem_set(data + 1, data + 5, 0x5A);
+      etl::mem_set(data + 1, data + 5, (char)0x5A);
 
       CHECK(std::equal(expected, expected + 8, data));
     }
@@ -1353,7 +1357,7 @@ namespace
       uint32_t data[8] = { 0, 0, 0, 0, 0, 0, 0, 0 };
       uint32_t expected[8] = { 0, 0x5A5A5A5A, 0x5A5A5A5A, 0x5A5A5A5A, 0x5A5A5A5A, 0, 0, 0 };
 
-      etl::mem_set(data + 1, 4, 0x5A);
+      etl::mem_set(data + 1, 4, (char)0x5A);
 
       CHECK(std::equal(expected, expected + 8, data));
     }
@@ -1363,11 +1367,18 @@ namespace
     {
       uint32_t data[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67294501, 0x45016723, 0x01324576, 0x76453201 };
 
-      char *p1 = etl::mem_char(data, data + 8, 0x29);
-      char* p2 = etl::mem_char(data, data + 8, 0x99);
+      char *p1 = etl::mem_char(data, data + 8, (char)0x29);
+      char* p2 = etl::mem_char(data, data + 8, (char)0x99);
 
       CHECK_EQUAL(uint32_t(0x29), uint32_t(*p1));
-      CHECK((reinterpret_cast<char*>(data) + 18) == p1);
+      if (etl::endianness::value() == etl::endian::little)
+      {
+        CHECK((reinterpret_cast<char*>(data) + 18) == p1);
+      }
+      else
+      {
+        CHECK((reinterpret_cast<char*>(data) + 17) == p1);
+      }
       CHECK((reinterpret_cast<char*>(data) + 32) == p2);
     }
 
@@ -1376,11 +1387,18 @@ namespace
     {
       const uint32_t data[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67294501, 0x45016723, 0x01324576, 0x76453201 };
 
-      const char* p1 = etl::mem_char(data, data + 8, 0x29);
-      const char* p2 = etl::mem_char(data, data + 8, 0x99);
+      const char* p1 = etl::mem_char(data, data + 8, (char)0x29);
+      const char* p2 = etl::mem_char(data, data + 8, (char)0x99);
 
       CHECK_EQUAL(uint32_t(0x29), uint32_t(*p1));
-      CHECK((reinterpret_cast<const char*>(data) + 18) == p1);
+      if (etl::endianness::value() == etl::endian::little)
+      {
+        CHECK((reinterpret_cast<const char*>(data) + 18) == p1);
+      }
+      else
+      {
+        CHECK((reinterpret_cast<const char*>(data) + 17) == p1);
+      }
       CHECK((reinterpret_cast<const char*>(data) + 32) == p2);
     }
 
@@ -1389,11 +1407,18 @@ namespace
     {
       uint32_t data[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67294501, 0x45016723, 0x01324576, 0x76453201 };
 
-      char* p1 = etl::mem_char(data, 8, 0x29);
-      char* p2 = etl::mem_char(data, 8, 0x99);
+      char* p1 = etl::mem_char(data, 8, (char)0x29);
+      char* p2 = etl::mem_char(data, 8, (char)0x99);
 
       CHECK_EQUAL(uint32_t(0x29), uint32_t(*p1));
-      CHECK((reinterpret_cast<char*>(data) + 18) == p1);
+      if (etl::endianness::value() == etl::endian::little)
+      {
+        CHECK((reinterpret_cast<char*>(data) + 18) == p1);
+      }
+      else
+      {
+        CHECK((reinterpret_cast<char*>(data) + 17) == p1);
+      }
       CHECK((reinterpret_cast<char*>(data) + 32) == p2);
     }
 
@@ -1402,11 +1427,18 @@ namespace
     {
       const uint32_t data[8] = { 0x12345678, 0x76543210, 0x01452367, 0x23670145, 0x67294501, 0x45016723, 0x01324576, 0x76453201 };
 
-      const char* p1 = etl::mem_char(data, 8, 0x29);
-      const char* p2 = etl::mem_char(data, 8, 0x99);
+      const char* p1 = etl::mem_char(data, 8, (char)0x29);
+      const char* p2 = etl::mem_char(data, 8, (char)0x99);
 
       CHECK_EQUAL(uint32_t(0x29), uint32_t(*p1));
-      CHECK((reinterpret_cast<const char*>(data) + 18) == p1);
+      if (etl::endianness::value() == etl::endian::little)
+      {
+        CHECK((reinterpret_cast<const char*>(data) + 18) == p1);
+      }
+      else
+      {
+        CHECK((reinterpret_cast<const char*>(data) + 17) == p1);
+      }
       CHECK((reinterpret_cast<const char*>(data) + 32) == p2);
     }
 
